import greenfoot.*;  // (World, Actor, GreenfootImage, and Greenfoot)

/**
 * The AnimatedActor class serves as a superclass for other, more specific
 * Actor classes. It provides a mechanism to give actors an animated image.
 * 
 * @author mik
 * @version 0.0.2
 */
public class AnimatedActor extends Actor
{
    private GreenfootImage[] images;
    protected float currentImage = 0F;
    private float animationSpeed = 0.1F;
    private int noOfImages;
    
    public AnimatedActor(String basename, String suffix, int noOfImages)
    {
        this.noOfImages = noOfImages;
        images = new GreenfootImage[noOfImages];
        for(int i=0; i < noOfImages; i++)
        {
            images[i] = new GreenfootImage(basename + i + suffix);
        }
        setImage(images[((int) currentImage)]);
    }
    
    /**
     * Do whatever the AnimatedActor wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act() 
    {
        currentImage = (currentImage + animationSpeed) % images.length;
        setImage(images[((int) currentImage)]);
    }
    
    /**
     * mirror ALL the images horizontally.
     */
    public void imagesMirrorHorizontally()
    {
        for (int i=0; i < noOfImages; i++)
        {
            images[i].mirrorHorizontally();
        }
    }
}
