import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * Clown that must pop balloons in this game.
 * 
 * @author Folkert van Verseveld 
 * @version 0.0.2
 */
public class Clown  extends Actor
{
    private static final int WIDTH = 32;
    private static final int HEIGHT = 32;
    private static final float gravitySpeed = 0.3F;
    private static final int maxSpeed = 16;
    private float vSpeed = 0F; // vSpeed = vertical speed
    private float hSpeed = 0F; // hSpeed = horizontal speed
    GreenfootImage body = new GreenfootImage("images/clowns/body.png");
    private GreenfootImage[] rArm; // rArm = right arm (images)
    private GreenfootImage[] lArm; // lArm = left arm (images)
    private float rArmCurrentImage = 0F;
    private float lArmCurrentImage = 0F;
    private static final float animArmSpeed = 0.05F;
    private static int noOfArm = 3;
    private GreenfootImage[] rLeg; // rLeg = right leg (images)
    private GreenfootImage[] lLeg; // lLeg = left leg (images)
    private float rLegCurrentImage = 0F;
    private float lLegCurrentImage = 0F;
    private static final float animLegSpeed = 0.05F;
    private static int noOfLeg = 2;
    private boolean disabled = false;
    private boolean dead = false;
    private int deadEllapsedTime = 0;
    private boolean gravityDisabled = false;
    private boolean canPop = true;
    private int canPopWait = 5;
    
    public Clown(boolean disabled)
    {
        this.disabled = disabled;
        setImage(new GreenfootImage(WIDTH, HEIGHT));
        GreenfootImage image = getImage();
        makeArms();
        makeLegs();
        updateImage();
    }
    
    /**
     * Imports the arm images
     */
    public void makeArms()
    {
        rArm = new GreenfootImage[noOfArm];
        lArm = new GreenfootImage[noOfArm];
        for(int i=0; i < noOfArm; i++)
        {
            rArm[i] = new GreenfootImage("images/clowns/arm-" + i + ".png");
            lArm[i] = new GreenfootImage("images/clowns/arm-" + i + ".png");
            lArm[i].mirrorHorizontally();
        }
    }
    
    /**
     * Imports the leg images
     */
    public void makeLegs()
    {
        rLeg = new GreenfootImage[noOfLeg];
        lLeg = new GreenfootImage[noOfLeg];
        for(int i=0; i < noOfLeg; i++)
        {
            rLeg[i] = new GreenfootImage("images/clowns/leg-" + i + ".png");
            lLeg[i] = new GreenfootImage("images/clowns/leg-" + i + ".png");
            lLeg[i].mirrorHorizontally();
        }
    }
    
    /**
     * Do whatever the Clown wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act() 
    {
        if (!disabled)
        {
            if (!gravityDisabled)
            {
                gravity();
            }
            if (!canPop)
            {
                canPopWait--;
                if (canPopWait <= 0)
                {
                    canPopWait = 5;
                    canPop = true;
                }
            }
            if (dead)
            {
                if (deadEllapsedTime < 120)
                {
                    deadEllapsedTime++;
                }
                else
                {
                    if (((MenuWorld)getWorld()).livesCounter.getValue() > 0)
                    {
                        resetVars();
                        resetPosition();
                        smallJump();
                    }
                }
            }
            if (getX() < 4 || getX() > ((MenuWorld)getWorld()).getWorldWidth() - 4)
            {
                hSpeed = -hSpeed;
                int i = (getX() < 4) ? 2 : -2;
                setLocation(getX() + i, getY());
            }
            if (getY() < 4)
            {
                vSpeed = -(vSpeed / 2);
                setLocation(getX(), 4);
            }
            animArms();
            animLegs();
            updateImage();
        }
        setLocation(getX() + ((int) hSpeed), getY() + ((int) vSpeed));
        int i = ((MenuWorld)getWorld()).getWorldHeight();
        if (getY() > i - 2 && !disabled && !dead)
        {
            deadClown(i);
        }
        
        TeeterTotter tT = (TeeterTotter) getOneIntersectingObject(TeeterTotter.class);
        if (tT != null && !disabled && !dead)
        {
            tT.launchClowns();
        }
    }
    
    /**
     * Runs when clown is dead
     */
    public void deadClown(int i)
    {
        dead = true;
        gravityDisabled = true;
        resetSpeeds();
        setLocation(getX(), i);
        mirrorClownVertically();
        updateImage();
        deadEllapsedTime++;
        ((MenuWorld)getWorld()).livesCounter.subtract(1);
        Greenfoot.playSound("sounds/splat.wav");
    }
    
    /**
     * Resets hSpeed and vSpeed
     */
    public void resetSpeeds()
    {
        hSpeed = 0;
        vSpeed = 0;
    }
    
    /**
     * Sets hSpeed to a float
     */
    public void setHSpeed(float hSpeed)
    {
        this.hSpeed = hSpeed;
    }
    
    /**
     * Returns hSpeed in a float
     */
    public float getHSpeed()
    {
        return hSpeed;
    }
    
    /**
     * Sets vSpeed to a float
     */
    public void setVSpeed(float vSpeed)
    {
        this.vSpeed = vSpeed;
    }
    
    /**
     * Returns vSpeed in a float
     */
    public float getVSpeed()
    {
        return vSpeed;
    }
    
    /**
     * Let the arms animate
     */
    public void animArms()
    {
        rArmCurrentImage = (rArmCurrentImage + animArmSpeed) % rArm.length;
        lArmCurrentImage = rArm.length - rArmCurrentImage;
    }
    
    /**
     * Let the legs animate
     */
    public void animLegs()
    {
        rLegCurrentImage = (rLegCurrentImage + animLegSpeed) % rLeg.length;
        lLegCurrentImage = rLeg.length - rLegCurrentImage;
    }
    
    /**
     * Makes gravity by clown
     */
    public void gravity()
    {
        if (vSpeed < maxSpeed)
        {
            vSpeed += gravitySpeed;
        }
        else if (vSpeed > maxSpeed)
        {
            vSpeed = maxSpeed;
        }
    }
    
    /**
     * Make the image
     */
    public void updateImage()
    {
        GreenfootImage image = getImage();
        image.clear();
        image.drawImage(body, 0, 0);
        image.drawImage(rArm[(int) rArmCurrentImage], 0, 0);
        image.drawImage(lArm[(int) lArmCurrentImage], 0, 0);
        image.drawImage(rLeg[(int) rLegCurrentImage], 0, 0);
        image.drawImage(lLeg[(int) lLegCurrentImage], 0, 0);
    }
    
    /**
     * Bounce
     */
    public void bounce()
    {
        if (vSpeed > 12)
        {
            vSpeed = -12;
            hSpeed = (getX() < 80) ? 5 : -5;
        }
        else
        {
            vSpeed = -8;
            hSpeed = (getX() < 80) ? 4 : -4;
        }
    }
    
    /**
     * Launch from the teeterTotter with a random vSpeed and hSpeed.
     */
    public void launch(boolean soundCount)
    {
        int i = Greenfoot.getRandomNumber(7);
        int j;
        switch(i)
        {
            case 0:
            case 4:
                j = Greenfoot.getRandomNumber(2);
                hSpeed = (j > 0) ? 2 : -2;
                vSpeed = -12;
                break;
            case 2:
            case 5:
                j = Greenfoot.getRandomNumber(2);
                hSpeed = (j > 0) ? 3 : -3;
                vSpeed = -14 + Greenfoot.getRandomNumber(2);
                break;
            default:
                vSpeed = -18 + Greenfoot.getRandomNumber(4);
                break;
        }
        boolean r = soundCount;
        if (r)
        {
            Greenfoot.playSound("sounds/teeter2.wav");
        }
        else
        {
            Greenfoot.playSound("sounds/teeter1.wav");
        }
    }
    
    /**
     * Sets disabled to boolean
     */
    public void setDisabled(boolean disabled)
    {
        this.disabled = disabled;
    }
    
    /**
     * Returns boolean if disabled
     */
    public boolean getDisabled()
    {
        return disabled;
    }
    
    /**
     * Mirrors the clown vertically
     */
    public void mirrorClownVertically()
    {
        body.mirrorVertically();
        for (int i=0; i < noOfArm; i++)
        {
            rArm[i].mirrorVertically();
            lArm[i].mirrorVertically();
        }
        for (int i=0; i < noOfLeg; i++)
        {
            rLeg[i].mirrorVertically();
            lLeg[i].mirrorVertically();
        }
    }
    
    /**
     * Set the images when the clown is on the teeter-totter
     */
    public void setImagesTeeterTotter()
    {
        rArmCurrentImage = 1;
        lArmCurrentImage = 1;
        rLegCurrentImage = 1;
        lLegCurrentImage = 1;
    }
    
    /**
     * Resets the vars of clown
     */
    public void resetVars()
    {
        rArmCurrentImage = 0;
        lArmCurrentImage = 0;
        rLegCurrentImage = 0;
        lLegCurrentImage = 0;
        dead = false;
        disabled = false;
        gravityDisabled = false;
        deadEllapsedTime = 0;
        mirrorClownVertically();
    }
    
    /**
     * Resets the position of clown. He'll 'jump' to the left or right side of
     * the screen.
     */
    public void resetPosition()
    {
        int i = Greenfoot.getRandomNumber(2);
        if (i > 0)
        {
            setLocation(16, 160);
        }
        else
        {
            setLocation(((MenuWorld)getWorld()).getWorldWidth() - 16, 160);
        }
    }
    
    /**
     * A small jump when clown starts fall
     */
    public void smallJump()
    {
        vSpeed = -4;
    }
    
    /**
     * Returns dead boolean
     */
    public boolean getDead()
    {
        return dead;
    }
    
    /**
     * Sets the canPopBoolean
     */
    public void setCanPop(boolean canPop)
    {
        this.canPop = canPop;
    }
    
    /**
     * Returns the canPop boolean
     */
    public boolean getCanPop()
    {
        return canPop;
    }
}
