import greenfoot.*;  // (World, Actor, GreenfootImage, and Greenfoot)
import java.awt.Color;
import java.awt.Font;

/**
 * Counter that displays the score / a number.
 * 
 * @author Folkert van Verseveld
 * @version 0.0.2
 */
public class Counter extends Actor
{
    private int value = 0;
    private int target = 0;
    private String text, sClownImage, sTimesImage;
    private int stringLength;
    private static final int WIDTH = 160;
    private static final int HEIGHT = 48;
    private static final float FONT_SIZE = 16.0F;
    private int x = 0;
    GreenfootImage image, clownImage, timesImage;

    public Counter()
    {
        this("", "", "", 0);
    }

    public Counter(String prefix, String sClownImage, String sTimesImage, int x)
    {
        text = prefix;
        stringLength = (text.length() + 2) * 16;
        this.sClownImage = sClownImage;
        this.sTimesImage = sTimesImage;
        this.x = x;
        
        makeImage();
        //Font font = image.getFont();
        //image.setFont(font.deriveFont(16.0F));
        
        updateImage();
    }
    
    public void act() {
        if(value < target) {
            value++;
            updateImage();
        }
        else if(value > target) {
            value--;
            updateImage();
        }
    }

    public void add(int score)
    {
        target += score;
    }

    public void subtract(int score)
    {
        target -= score;
    }

    public int getValue()
    {
        return value;
    }
    
    public int getTarget()
    {
        return target;
    }
    
    public void setScore(int score)
    {
        target = score;
        value = score;
        updateImage();
    }
    
    /**
     * Make the image
     */
    public void makeImage()
    {
        image = new GreenfootImage(WIDTH, HEIGHT);
        if (sClownImage != "")
        {
            clownImage = new GreenfootImage(sClownImage);
        }
        if (sTimesImage != "")
        {
            timesImage = new GreenfootImage(sTimesImage);
        }
        
        updateImage();
        //image.drawString("Press <space> to restart", 60, 256);
        setImage(image);
    }

    /**
     * Update the image
     */
    private void updateImage()
    {
        //GreenfootImage image = getImage();
        image.clear();
        image.setColor(new Color(0, 0, 0, 160));
        image.fillRect(0, 0, WIDTH, HEIGHT);
        image.setColor(new Color(255, 255, 255, 100));
        image.fillRect(5, 5, WIDTH-10, HEIGHT-10);
        Font font = image.getFont();
        font = font.deriveFont(FONT_SIZE);
        image.setFont(font);
        image.setColor(Color.WHITE);
        //image.drawString(title, 60, 100);
        if (clownImage != null)
        {
            image.drawImage(clownImage, 8, 8);
        }
        if (timesImage != null)
        {
            image.drawImage(timesImage, 32, 8);
        }
        image.drawString(text + value, 32 + x, 32);
        //font = font.deriveFont(FONT_SIZE_SMALL);
        //image.setFont(font);
        //image.drawString(text + value, 1, 18);
    }
    
    /**
     * Resets the values of counter
     */
    public void reset()
    {
        value = 0;
        target = 0;
        updateImage();
    }
}

/*public static final float FONT_SIZE = 48.0f;
    public static final float FONT_SIZE_SMALL = 16.0f;
    public static final int WIDTH = 400;
    public static final int HEIGHT = 300;*/
