import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

//import mymodplayer.*;   // My code to play mod-files.

/**
 * Write a description of class MenuWorld here.
 * 
 * @author Folkert van Verseveld 
 * @version 0.0.2
 */
public class MenuWorld  extends World
{
    // world images and image floats
    private GreenfootImage[] images;
    //private static GreenfootSound gfs = null;
    private static final GreenfootImage emptyBackground = new GreenfootImage("images/status/black.png");
    private float currentImage = 0F;
    private float animationSpeed = 0.1F;
    
    // width and height
    private static final int MENU_WORLD_WIDTH = 640;
    private static final int MENU_WORLD_HEIGHT = 480;
    
    private static boolean renewRow[] = new boolean[3];
    private static final int renewY = 128; // if both the clowns have a bigger
    // y than the renewY and one or more row(s) is/are empty; the world will
    // create a new random balloon row.
    public static int rowCount[] = new int[3];
    private Clown clown,worldClown;
    public static Counter counter, livesCounter;
    private int score;
    
    // Dificult booleans
    public boolean barriers = false;
    
    // Initialize the worlds
    private static final int MENU = 1;
    private static final int PLAY = 2;
    private static final int DEAD = 3;
    private static int generatedWorld = MENU; // the world that must be
                                              // generated.
    
    /**
     * Constructor for objects of class MenuWorld.
     * 
     */
    public MenuWorld()
    {
        // Create a new world with 640x480 cells with a cell size of 1x1 pixel.
        super(MENU_WORLD_WIDTH, 480, 1);
        for(int i = 0; i < rowCount.length; i++)
        {
            rowCount[i] = MENU_WORLD_WIDTH / 32;
        }
        for(int i = 0; i < renewRow.length; i++)
        {
            renewRow[i] = false;
        }
        worldInit();
    }
    
    /**
     * Checks in which world we are and create the world.
     */
    public void worldInit()
    {
        switch(generatedWorld)
        {
            case MENU:
                generateMenuWorld();
                break;
            case PLAY:
                generatePlayWorld();
                break;
            case DEAD:
                generateDeadWorld();
                break;
        }
        //playMusic();
    }
    
    /**
     * Create the menu world
     */
    public void generateMenuWorld()
    {
        setPaintOrder(Balloon.class, Light.class, MenuButton.class);
        images = new GreenfootImage[1];
        images[0] = new GreenfootImage("images/title/title.png");
        Light light = new Light(0);
        addObject(light, 8, 384);
        addObject(new OnePlayerButton(), 128, 292);
        addObject(new GameExitButton(), 585, 443);
        addObject(new BarrierButton(light), 112, 384);
        // create 16 balloons
        for(int i = 0; i < 16; i++)
        {
            addObject(new MenuBalloon(),
                      8 + Greenfoot.getRandomNumber(MENU_WORLD_WIDTH - 8),
                      8 + Greenfoot.getRandomNumber(MENU_WORLD_HEIGHT - 8));
        }
        Greenfoot.start();
    }
    
    /**
     * Create the normal 'play' world.
     */
    public void generatePlayWorld()
    {
        setPaintOrder(Counter.class, Balloon.class, TeeterTotter.class, Clown.class, Barrier.class, Bouncer.class, Bear.class, Seal.class, Beachball.class);
        addAct();
        worldClown = new Clown(false);
        addObject(worldClown, 16, 160);
        worldClown.smallJump();
        clown = new Clown(true);
        counter = new Counter("Score: ", "", "", 0);
        livesCounter = new Counter("", "images/status/clown-head.png", "images/status/times.png", 32);
        addObject(counter, 80, 32);
        addObject(livesCounter, MENU_WORLD_WIDTH - 80, 32);
        livesCounter.add(5);
        addObject(clown, MENU_WORLD_WIDTH / 2 - 32, MENU_WORLD_HEIGHT - 8);
        addObject(new Bouncer(), 16, MENU_WORLD_HEIGHT - 16);
        addObject(new Bouncer(), MENU_WORLD_WIDTH - 16, MENU_WORLD_HEIGHT - 16);
        addObject(new TeeterTotter(clown, worldClown), MENU_WORLD_WIDTH / 2, MENU_WORLD_HEIGHT - 16);
        if (barriers)
        {
            addObject(new Barrier(), 128, 112);
            addObject(new Barrier(), 256, 112);
            addObject(new Barrier(), 384, 112);
        }
        // Add a row of blue balloons to test.
        for (int i = 0; i < rowCount[0]; i++)
        {
            addObject(new RedBalloon(0), 16 + i * 32, 16);
            addObject(new YellowBalloon(1), i * 32, 48);
            addObject(new CyanBalloon(2), 16 + i * 32, 80);
        }
        
        images = new GreenfootImage[2];
        for(int i=0; i < 2; i++)
        {
            images[i] = new GreenfootImage("images/backgrounds/background" + i + ".png");
        }
        setBackground(images[((int) currentImage)]);
    }
    
    /**
     * Create the deadWorld when your lives <= 0
     */
    public void generateDeadWorld()
    {
        barriers = false;
        addObject(new DeadClown(), MENU_WORLD_WIDTH - 64, MENU_WORLD_HEIGHT - 80);
        Counter scoreShower = new Counter("SCORE: ", "", "", -8);
        scoreShower.setScore(score);
        addObject(scoreShower, MENU_WORLD_WIDTH / 2, 80);
        setBackground(emptyBackground);
    }
    
    /**
     * Returns the created clown instance.
     */
    public Clown getClown()
    {
        return clown;
    }
    
    /**
     * Adds a random act.
     */
    public void addAct()
    {
        int randomAct = Greenfoot.getRandomNumber(2);
        switch(randomAct)
        {
            case 0:
                addObject(new Seal(), 180, 372);
                addObject(new Beachball(), 166, 256);
                break;
            default:
                addObject(new Bear(), 160, 384);
                break;
        }
    }
    
    /**
     * The 'animate' event for the world.
     */
    public void act()
    {
        try {
        switch(generatedWorld)
        {
            case MENU:
                setBackground(images[0]);
                break;
            case PLAY:
                currentImage = (currentImage + animationSpeed) % images.length;
                setBackground(images[((int) currentImage)]);
                int i = renewY;
                if (renewRow[0])
                {
                    if (clown.getY() > i && worldClown.getY() > i)
                    {
                        createRandomBalloonRow(0);
                    }
                }
                if (renewRow[1])
                {
                    if (clown.getY() > i && worldClown.getY() > i)
                    {
                        createRandomBalloonRow(1);
                    }
                }
                if (renewRow[2])
                {
                    if (clown.getY() > i && worldClown.getY() > i)
                    {
                        createRandomBalloonRow(2);
                    }
                }
                if (livesCounter.getTarget() <= 0)
                {
                    gotoWorld(DEAD);
                }
                break;
            case DEAD:
                if (Greenfoot.mouseClicked(null))
                {
                    gotoWorld(MENU);
                }
                break;
            default:
                setBackground(emptyBackground);
                break;
        }
    } catch (Exception e) {
        System.out.println("MenuWorld exception caught: e="+e);
    }
    }
    
    /**
     * Returns the width of the menuWorld
     */
    public int getWorldWidth()
    {
        return MENU_WORLD_WIDTH;
    }
    
    /**
     * Returns the height of the menuWorld
     */
    public int getWorldHeight()
    {
        return MENU_WORLD_HEIGHT;
    }
    
    /**
     * Check the parameter row if the counter equals to 0. If the counter
     * equals to 0, the world will automaticly create a new random balloon row.
     */
    public void checkRow(int row)
    {
        if (rowCount[row] <= 0)
        {
            // now we create a new random balloon row.
            rowCount[row] = MENU_WORLD_WIDTH / 32;
            renewRow[row] = true;
        }
    }
    
    /**
     * Create a new random balloon row
     */
    public void createRandomBalloonRow(int row)
    {
        renewRow[row] = false;
//        newRow = 0;
        int i = Greenfoot.getRandomNumber(8);
        int j;
        int r = (row == 1) ? 0 : 16;
        int n = MENU_WORLD_WIDTH / 32;
        switch(i)
            {
            case 0:
                for(j = 0; j < n; j++)
                {
                    addObject(new YellowBalloon(row), r + j * 32, 16 + row * 32);
                }
                break;
            case 1:
                for(j = 0; j < n; j++)
                {
                    addObject(new RedBalloon(row), r + j * 32, 16 + row * 32);
                }
                break;
            case 2:
                for(j = 0; j < n; j++)
                {
                    addObject(new CyanBalloon(row), r + j * 32, 16 + row * 32);
                }
                break;
            case 3:
                for(j = 0; j < n; j++)
                {
                    addObject(new BlueBalloon(row), r + j * 32, 16 + row * 32);
                }
                break;
            case 4:
                for(j = 0; j < n; j++)
                {
                    addObject(new GreenBalloon(row), r + j * 32, 16 + row * 32);
                }
                break;
            case 5:
                for(j = 0; j < n; j++)
                {
                    addObject(new OrangeBalloon(row), r + j * 32, 16 + row * 32);
                }
                break;
            case 6:
                for(j = 0; j < n; j++)
                {
                    addObject(new PurpleBalloon(row), r + j * 32, 16 + row * 32);
                }
                break;
            default:
                for(j = 0; j < n; j++)
                {
                    addObject(new WhiteBalloon(row), r + j * 32, 16 + row * 32);
                }
                break;
        }
        counter.add(100);
        Greenfoot.playSound("sounds/applause.wav");
    }
    
    /**
     * Goto next world
     */
    public void nextWorld()
    {
        removeObjects(getObjects(null));
        switch(generatedWorld)
        {
            case MENU:
                generatedWorld = PLAY;
                worldInit();
                break;
            default:
                generatedWorld = MENU;
                worldInit();
                break;
        }
    }
    
    /**
     * Goto the parameter world
     * @param world the 'goto' world
     */
    public void gotoWorld(int world)
    {
        if (world == DEAD)
        {
            score = counter.getTarget();
        }
        generatedWorld = world;
        removeObjects(getObjects(null));
        worldInit();
    }
    
    /*public void started()
    {
        if (generatedWorld == MENU && gfs == null)
        {
            playMusic();
        }
    }
    
    //public void stopped(){
    //    stopSound();
    //}
    
    public void playSound(String soundFile)
    {
        try
        {
            if (gfs != null)
            {
                gfs.stopSound();
            }
            gfs = new GreenfootSound(soundFile);
            gfs.playSound();
        }
        catch (Exception e)
        {
            System.out.println("playSound() -> e="+e);
        }
    }
    
    public void stopSound()
    {
        try
        {
            if (gfs != null)
            {
                gfs.stopSound();
                gfs = null;
            }
        }
        catch (Exception e)
        {
            System.out.println("stopSound() -> e="+e);
        }
    }
    
    public void playMusic()
    {
        switch(generatedWorld)
        {
            case MENU:
                playSound("sounds/music/finally.mod");
                break;
            case PLAY:
                playSound("sounds/music/klovninarki.mod");
                break;
            case DEAD:
                playSound("sounds/music/kaupunki.mod");
                break;
        }
    }*/
}
